' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   DIM   TextureHandles(2) AS DWORD   ' Storage for 3 textures
   LOCAL filter AS LONG               ' Which filter to use
   LOCAL light AS LONG                ' Lighting on/off
   LOCAL lp AS LONG                   ' L pressed?
   LOCAL fp AS LONG                   ' F pressed?
   LOCAL sp AS LONG                   ' Spacebar pressed?
   LOCAL part1 AS LONG                ' Start of disc
   LOCAL part2 AS LONG                ' End of disc
   LOCAL p1 AS LONG                   ' Increase 1
   LOCAL p2 AS LONG                   ' Increase 2
   LOCAL xrot AS SINGLE               ' X rotation
   LOCAL yrot AS SINGLE               ' Y rotation
   LOCAL xspeed AS SINGLE             ' X rotation speed
   LOCAL yspeed AS SINGLE             ' Y rotation speed
   LOCAL z AS SINGLE                  ' Depth into the screen
   LOCAL nObject AS DWORD             ' Which object to draw
   LOCAL quadratic AS DWORD           ' Quadratic

   DIM LightAmbient(3) AS SINGLE
   DIM LightDiffuse(3) AS SINGLE
   DIM LightPosition(3) AS SINGLE

   p1 = 0 : p2 = 1 : z = -5.0!

   ARRAY ASSIGN LightAmbient()  = 0.5!, 0.5!, 0.5!, 1.0!
   ARRAY ASSIGN LightDiffuse()  = 1.0!, 1.0!, 1.0!, 1.0!
   ARRAY ASSIGN LightPosition() = 0.0!, 0.0!, 2.0!, 1.0!

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("wall.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign handles
   glGenTextures 3, TextureHandles(0)

   ' Create Nearest Filtered Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(0)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_NEAREST
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_NEAREST
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create Linear Filtered Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(1)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create MipMapped Texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(2)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR_MIPMAP_NEAREST
   gluBuild2DMipmaps %GL_TEXTURE_2D, 3, TextureWidth, TextureHeight, _
                     %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Enable texture mapping
   glEnable %GL_TEXTURE_2D
   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' The type of depth test to do
   glDepthFunc %GL_LEQUAL
   ' Enables depth testing
   glEnable %GL_DEPTH_TEST
	' Really nice perspective calculations
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST

   glLightfv %GL_LIGHT1, %GL_AMBIENT,  LightAmbient(0)  ' Setup the ambient light
   glLightfv %GL_LIGHT1, %GL_DIFFUSE,  LightDiffuse(0)  ' Setup the diffuse light
   glLightfv %GL_LIGHT1, %GL_POSITION, LightPosition(0) ' Position the light
   glEnable  %GL_LIGHT1                                 ' Enable light one

   quadratic = gluNewQuadric                            ' Create a pointer to the quadric object
   gluQuadricNormals quadratic, %GLU_SMOOTH             ' Create smooth normals
   gluQuadricTexture quadratic, %GL_TRUE                ' Create texture coords

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 18 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and the depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      glTranslatef 0.0!, 0.0!, z
      glRotatef xrot, 1.0!, 0.0!, 0.0!
      glRotatef yrot, 0.0!, 1.0!, 0.0!
      glBindTexture %GL_TEXTURE_2D, TextureHandles(filter)

      SELECT CASE nObject

         CASE 0
            glBegin %GL_QUADS
            ' Front face
            glNormal3f 0.0!, 0.0!, 1.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
            ' Back face
            glNormal3f 0.0!, 0.0!, -1.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
            glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
            ' Top face
            glNormal3f 0.0!, 1.0!, 0.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
            glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!,  1.0!,  1.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!,  1.0!,  1.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
            ' Bottom face
            glNormal3f 0.0!, -1.0!, 0.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!, -1.0!, -1.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!, -1.0!, -1.0!
            glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
            ' Right face
            glNormal3f 1.0!, 0.0!, 0.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
            glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
            ' Left face
            glNormal3f -1.0!, 0.0!, 0.0!
            glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
            glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
            glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
            glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
            glEnd

         CASE 1
            ' Center the cylinder
            glTranslatef 0.0!, 0.0!, -1.5!
            ' A cylinder with a radius of 0.5 and a height of 2
            gluCylinder quadratic, 1.0!, 1.0!, 3.0!, 32, 32

         CASE 2
            ' Draw a disc (cd shape) with an inner radius of 0.5, and an outer radius of 2.  plus a lot of segments ;)
            gluDisk quadratic, 0.5!, 1.5!, 32, 32

         CASE 3
            '  Draw a sphere with a radius of 1 and 16 longitude and 16 latitude segments
            gluSphere quadratic, 1.3!, 32, 32

         CASE 4
            '  Center the cone
            glTranslatef 0.0!, 0.0!, -1.5!
            '  a cone with a bottom radius of .5 and a height of 2
            gluCylinder quadratic, 1.0!, 0.0!, 3.0!, 32, 32

         CASE 5
            part1 = part1 + p1
            part2 = part2 + p2
            IF part1 > 359 THEN               ' 360 Degrees
               p1 = 0
               part1 = 0
               p2 = 1
               part2 = 0
            END IF
            IF part2 > 359 THEN               ' 360 Degrees
               p1 = 1
               p2 = 0
            END IF
            '  A disk like the one before
            gluPartialDisk quadratic, 0.5!, 1.5!, 32, 32, part1, part2 - part1

      END SELECT

      xrot = xrot + xspeed
      yrot = yrot + yspeed

      ' Swap buffers
      glfwSwapBuffers

      ' L key
      IF glfwGetKey(76) = %GLFW_PRESS AND ISFALSE lp THEN
         lp = %TRUE
         light = NOT light
         IF ISFALSE light THEN
            glDisable %GL_LIGHTING
         ELSE
            glEnable %GL_LIGHTING
         END IF
      END IF
      IF glfwGetKey(76) <> %GLFW_PRESS THEN lp = %FALSE

      ' F key
      IF glfwGetKey(70) = %GLFW_PRESS AND ISFALSE fp THEN
         fp = %TRUE
         filter = filter + 1
         IF filter > 2 THEN filter = 0
      END IF
      IF glfwGetKey(70) <> %GLFW_PRESS THEN fp = %FALSE

      ' Space
      IF glfwGetKey(32) = %GLFW_PRESS AND ISFALSE sp THEN
         sp = %TRUE
         nObject = nObject + 1
         IF nObject > 5 THEN nObject = 0
      END IF
      IF glfwGetKey(32) <> %GLFW_PRESS THEN sp = %FALSE

      IF glfwGetKey(%GLFW_KEY_PAGEUP) = %GLFW_PRESS THEN z = z - 0.02!
      IF glfwGetKey(%GLFW_KEY_PAGEDOWN) = %GLFW_PRESS THEN z = z + 0.02!
      IF glfwGetKey(%GLFW_KEY_UP) = %GLFW_PRESS THEN xspeed = xspeed - 0.01!
      IF glfwGetKey(%GLFW_KEY_DOWN) = %GLFW_PRESS THEN xspeed = xspeed + 0.01!
      IF glfwGetKey(%GLFW_KEY_RIGHT) = %GLFW_PRESS THEN yspeed = yspeed + 0.01!
      IF glfwGetKey(%GLFW_KEY_LEFT) = %GLFW_PRESS THEN yspeed = yspeed - 0.01!

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the quadratic to free system resources
   gluDeleteQuadric quadratic

   ' Delete the textures
   glDeleteTextures(3, TextureHandles(0))

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
